@@ -22,7 +22,6 @@ use {
     std::{
         cmp::Ordering,
         collections::HashSet,
-        convert::TryFrom,
         fmt::{Debug, Formatter},
         hash::{Hash, Hasher},
         io::Write,
@@ -113,7 +112,7 @@ impl X509Certificate {
         data: impl AsRef<[u8]>,
         tags: &[&str],
     ) -> Result<Vec<Self>, Error> {
-        let pem = pem::parse_many(data.as_ref());
+        let pem = pem::parse_many(data.as_ref()).map_err(Error::PemDecode)?;
 
         pem.into_iter()
             .filter(|pem| tags.contains(&pem.tag.as_str()))
@@ -219,17 +218,30 @@ impl X509Certificate {
             .algorithm
     }
 
+    /// Obtain the [SignatureAlgorithm this certificate will use.
+    ///
+    /// Returns [None] if we failed to resolve an instance (probably because we don't
+    /// recognize the algorithm).
+    pub fn signature_algorithm(&self) -> Option<SignatureAlgorithm> {
+        SignatureAlgorithm::try_from(&self.0.tbs_certificate.signature.algorithm).ok()
+    }
+
+    /// Obtain the OID of the signature algorithm this certificate will use.
+    pub fn signature_algorithm_oid(&self) -> &Oid {
+        &self.0.tbs_certificate.signature.algorithm
+    }
+
     /// Obtain the [SignatureAlgorithm] used to sign this certificate.
     ///
     /// Returns [None] if we failed to resolve an instance (probably because we
     /// don't recognize that algorithm).
-    pub fn signature_algorithm(&self) -> Option<SignatureAlgorithm> {
+    pub fn signature_signature_algorithm(&self) -> Option<SignatureAlgorithm> {
         SignatureAlgorithm::try_from(&self.0.signature_algorithm).ok()
     }
 
     /// Obtain the OID of the signature algorithm used to sign this certificate.
-    pub fn signature_algorithm_oid(&self) -> &Oid {
-        &self.0.tbs_certificate.signature.algorithm
+    pub fn signature_signature_algorithm_oid(&self) -> &Oid {
+        &self.0.signature_algorithm.algorithm
     }
 
     /// Obtain the raw data constituting this certificate's public key.
@@ -400,7 +412,7 @@ impl CapturedX509Certificate {
         data: impl AsRef<[u8]>,
         tags: &[&str],
     ) -> Result<Vec<Self>, Error> {
-        let pem = pem::parse_many(data.as_ref());
+        let pem = pem::parse_many(data.as_ref()).map_err(Error::PemDecode)?;
 
         pem.into_iter()
             .filter(|pem| tags.contains(&pem.tag.as_str()))
@@ -451,6 +463,24 @@ impl CapturedX509Certificate {
         self.verify_signed_by_public_key(public_key)
     }
 
+    /// Verify a signature over signed data perportedly signed by this certificate.
+    pub fn verify_signed_data(
+        &self,
+        signed_data: impl AsRef<[u8]>,
+        signature: impl AsRef<[u8]>,
+    ) -> Result<(), Error> {
+        let key_algorithm = KeyAlgorithm::try_from(self.key_algorithm_oid())?;
+        let signature_algorithm = SignatureAlgorithm::try_from(self.signature_algorithm_oid())?;
+        let verify_algorithm = signature_algorithm.resolve_verification_algorithm(key_algorithm)?;
+
+        let public_key =
+            signature::UnparsedPublicKey::new(verify_algorithm, self.public_key_data());
+
+        public_key
+            .verify(signed_data.as_ref(), signature.as_ref())
+            .map_err(|_| Error::CertificateSignatureVerificationFailed)
+    }
+
     /// Verifies that this certificate was cryptographically signed using raw public key data from a signing key.
     ///
     /// This function does the low-level work of extracting the signature and
@@ -496,7 +526,7 @@ impl CapturedX509Certificate {
         let public_key = signature::UnparsedPublicKey::new(verify_algorithm, public_key_data);
 
         public_key
-            .verify(&signed_data, &signature)
+            .verify(signed_data, &signature)
             .map_err(|_| Error::CertificateSignatureVerificationFailed)
     }
 
